/***************************************************************************
 *   Copyright (C) 2006 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "phonemodelloader.h"

#include <qfile.h>
#include <qdatetime.h>
#include <qdom.h>
#include <qregexp.h>

#include <klocale.h>
#include <kstddirs.h>
#include <kmessagebox.h>


PhoneModelLoader::PhoneModelLoader(QWidget *parent)
{
  QString filename = locate("appdata", "phonemodels.xml");
  m_document = new QDomDocument( "tkphonemodels" );
  QFile file( filename );
  if ( !file.open( IO_ReadOnly ) ) {
    KMessageBox::sorry(parent, i18n("Could not read the phone models configuration. Aborting."));
    exit(0);
    return;
  }
  QString * errorMsg = new QString();// = new QString();
  int * errorLine = new int();
  int * errorCol = new int();
  if ( !m_document->setContent( &file, errorMsg, errorLine , errorCol ) ) {
      file.close();
      KMessageBox::sorry(parent, i18n("Could not load the phone models configuration. Error %1 in line %2, column %3. Aborting.").arg(*errorMsg).arg(*errorLine).arg(*errorCol));
      delete errorMsg;
      delete errorLine;
      delete errorCol;
      exit(0);
  }
  delete errorMsg;
  delete errorLine;
  delete errorCol;
  file.close();
}

QPtrList<PhoneModelGroup> * PhoneModelLoader::getGroups() {
  QPtrList<PhoneModelGroup> *groups;
  groups = new QPtrList<PhoneModelGroup>();
  groups->setAutoDelete( TRUE );
  
  QDomElement root = m_document->documentElement();
  QDomNode node;
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "group" ) {
      QString name = node.toElement().attribute( "name" );
      QString icon = node.toElement().attribute( "icon" );
      QString iconfile = locate("appdata", icon);
      groups->append(new PhoneModelGroup(name, QPixmap(iconfile)));
    }
    node = node.nextSibling();
  }
  return groups;
}


PhoneModelGroup *PhoneModelLoader::getGroup(const QString &version) {
  QDomElement root = m_document->documentElement();
  QDomNode node;
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "group" && node.toElement().attribute("version") == version ) {
      QString name = node.toElement().attribute( "name" );
      QString icon = node.toElement().attribute( "icon" );
      QString iconfile = locate("appdata", icon);
      return new PhoneModelGroup(name, QPixmap(iconfile));
    }
    node = node.nextSibling();
  }
  return 0;
}


QPtrList<PhoneModelItem> * PhoneModelLoader::getModels(const QString & group) {
  QPtrList<PhoneModelItem> *models;
  models = new QPtrList<PhoneModelItem>();
  models->setAutoDelete( TRUE );
  
  QDomElement root = m_document->documentElement();
  QDomNode group_node = root.firstChild();
  while ( !group_node.isNull() ) {
    if ( group_node.isElement() && group_node.nodeName() == "group" &&
      group_node.toElement().attribute( "name" ) == group )
    {
      QDomNode model_node = group_node.firstChild();
      while ( !model_node.isNull() ) {
        if ( model_node.isElement() && model_node.nodeName() == "model")
        {
          QString name = model_node.toElement().attribute( "name" );
          QString icon = model_node.toElement().attribute( "icon" );
          QString iconfile = locate("appdata", icon);
          QString description = QString::null;
          QDomNode descr_node = model_node.firstChild();
          while ( !descr_node.isNull() ) {
            if ( descr_node.isElement() && descr_node.nodeName() == "description" )
            {
              description = i18n(descr_node.toElement().firstChild().toText().nodeValue());
              break;
            }
            descr_node = descr_node.nextSibling();
          }
          models->append(new PhoneModelItem(name, QPixmap(iconfile), description));
        }
        model_node = model_node.nextSibling();
      }
    }
    group_node = group_node.nextSibling();
  }
  return models;
}


QString PhoneModelLoader::findModel(QString ett_model) {
  ett_model = ett_model.lower();
  //ett_model = ett_model.replace("x", "[0-9]" );
  ett_model = ett_model.replace(QRegExp("(k|v|d|w)"), "[a-z]" ); // v: vodaphone, d: t-mobile
  QRegExp modelRegExp = QRegExp(ett_model, FALSE, TRUE );

  QDomElement root = m_document->documentElement();
  QDomNode group_node = root.firstChild();
  while ( !group_node.isNull() ) {
    if ( group_node.isElement() && group_node.nodeName() == "group" )
    {
      QDomNode model_node = group_node.firstChild();
      while ( !model_node.isNull() ) {
        if ( model_node.isElement() && model_node.nodeName() == "model")
        {
          QString tk_model = model_node.toElement().attribute( "name" );
          if ( modelRegExp.search(tk_model) != -1 ) {
            return tk_model;
          }
        }
        model_node = model_node.nextSibling();
      }
    }
    group_node = group_node.nextSibling();
  }
  return "unknown";
}


PhoneModelLoader::~PhoneModelLoader()
{
  delete m_document;
}


